const { COMPONENT_TYPE } = require("../component/component_type")
const { SPECIAL_FIELDS } = require("../component/components")

const ROLE_TYPE = {
  USER: 3,
  PROJECT_ROLE: 1,
  SYSTEM_ROLE: 2,
}

const OWNER_TYPE = {
  OWNER: "owner",
  WORKER: "worker",
  OTHER: "other",
}
const OWNER_TYPES = ["other", "worker", "owner"]

/**
 *
 * @param {*}
 *   context = {
 *      item: { 任务信息
 *        owner: { id, name, type },  任务的负责人信息
 *        workers: [{ id, name, type }]  任务的参与人信息
 *      },
 *      currentUser: {  当前用户信息
 *        id, name,
 *        roles: [ { id, name, type } ] 当前用户在项目的角色信息
 *      }
 *   }  type（参见ROLE_TYPE）
 */
function calcUserOwnerTypes(context) {
  const { item, currentUser } = context
  const currentUserOwnerTypes = []

  if (currentUser && item) {
    const currentUserRoles = currentUser.roles || []

    const currentUserRoleMap = {
      [`${currentUser.id}-${ROLE_TYPE.USER}`]: 3,
    }

    currentUserRoles.forEach((r) => {
      currentUserRoleMap[`${r.id}-${r.type}`] = 1
    })

    if (item) {
      if (item.owner) {
        if (item.owner.type === ROLE_TYPE.USER) {
          if (item.owner.id === currentUser.id) {
            currentUserOwnerTypes.push(OWNER_TYPE.OWNER)
          }
        } else {
          if (currentUserRoleMap[`${item.owner.id}-${item.owner.type}`]) {
            currentUserOwnerTypes.push(OWNER_TYPE.OWNER)
          }
        }
      }

      if (
        item.workers &&
        item.workers.find((r) => currentUserRoleMap[`${r.id}-${r.type}`])
      ) {
        currentUserOwnerTypes.push(OWNER_TYPE.WORKER)
      }
    }
  }

  if (currentUserOwnerTypes.length === 0)
    currentUserOwnerTypes.push(OWNER_TYPE.OTHER)
  return currentUserOwnerTypes
}

/**
 *
 * @param {*} form: object 表单配置
 * @param {*} accessMap: object 权限配置 { owner, worker, other } https://captain.yuque.com/trv7ta/mu57ky/znpc53
 * @param {*}
 *   context = {
 *      item: { 任务信息
 *        owner: { id, name, type },  任务的负责人信息
 *        workers: [{ id, name, type }]  任务的参与人信息
 *      },
 *      currentUser: {  当前用户信息
 *        id, name,
 *        roles: [ { id, name, type } ] 当前用户在项目的角色信息
 *      }
 *   }  type（参见ROLE_TYPE）
 * @returns userAccess = {
    add_row: false,
    delete_row: false,
 *  fields: { [fieldId]: { readonly: boolean, visible: boolean } }
 * }
 */
function parseAccess(form, accessMap = {}, context = {}) {
  accessMap = accessMap || {}

  const { fields = [] } = form

  const currentUserOwnerTypes = calcUserOwnerTypes(context)
  let isOwner = currentUserOwnerTypes.find((v) => v === OWNER_TYPE.OWNER)
  let isWorker = currentUserOwnerTypes.find((v) => v === OWNER_TYPE.WORKER)
  const userAccess = {
    add_row: isOwner ? true : false,
    delete_row: isOwner ? true : false,
    fields: {},
  }
  if (currentUserOwnerTypes.length === 0)
    currentUserOwnerTypes.push(OWNER_TYPE.OTHER)

  fields.forEach((fieldConfig, index) => {
    let item = isOwner // 负责人
      ? { readonly: false, visible: true }
      : isWorker // 参与人
      ? { readonly: true, visible: true }
      : index === 0 // 标题默认只读
      ? { readonly: true, visible: true }
      : { readonly: true, visible: false }
    userAccess.fields[fieldConfig.id] = item
  })

  currentUserOwnerTypes.forEach((key) => {
    if (!accessMap[key]) return
    userAccess.add_row =
      userAccess.add_row || accessMap[key].add_row ? true : false
    userAccess.delete_row =
      userAccess.delete_row || accessMap[key].delete_row ? true : false
    Object.keys(accessMap[key].fields || {}).forEach((fid) => {
      const f = accessMap[key].fields[fid]
      const itemObj = userAccess.fields[fid]
      if (!itemObj) return

      itemObj.visible = itemObj.visible || f.visible ? true : false
      if (!itemObj.visible) {
        itemObj.readonly = true
        return
      }

      itemObj.readonly = !itemObj.readonly || !f.readonly ? false : true
    })
  })

  return userAccess
}

module.exports = {
  ROLE_TYPE,
  parseAccess,
  calcUserOwnerTypes,
}
