'use strict';
const dayjs = require('dayjs');
const { COMPONENT_TYPE } = require('../component/component_type');

/**
 * 
 * @param {*} form 表单信息
 * @param {*} datas form_datas数据
 * @param {*} orders 排序规则 [["f1", "asc"], ["f2", "desc"]]
 */
function parseDatasOrder(form, datas, orders) {
  let list = datas;
  for (let i = orders.length - 1; i > -1; i--) {
    const order = orders[i];
    list = parseOrder(form, list, order);
  }
  return list;
}

function parseOrder(form, datas = [], order = []) {
  const list = datas.sort((a, b) => {
    const key = order[0];
    const rule = order[1];
    const a_value = getFieldValue(form, key, a.form_datas);
    const b_value = getFieldValue(form, key, b.form_datas);
    let value = 0;
    switch (a_value.type) {
      case 'string':
        value = stringOrder(a_value.data, b_value.data);
        break;
      case 'number':
        value = numberOrder(a_value.data, b_value.data);
      default: value = stringOrder(a_value.data, b_value.data); 
        break;
    }
    value = orderParse(value, rule);
    return value;
  });
  return list;
}

function getFieldValue(form, fid, form_data) {
  let type = 'string';
  const field = form.setting.fields.find(f => f.id === fid);
  if (!field) return { data: null, type };
  let data = form_data[fid];
  if (!data) {
    if (data === 0) {
      data = 0;
    }
    data = null;
  } else {
    switch (field.controlType) {
      case COMPONENT_TYPE.SELECTCOLOR:
        data = getSelectValue(field, data);
        break;
      case COMPONENT_TYPE.MULTIPLE_SELECT:
        data = getMulipleSelectValue(field, data);
        break;
      case COMPONENT_TYPE.NUMBER:
        type = 'number';
        break;
      case COMPONENT_TYPE.DATE:
        data = getDateValue(field, data);
        break;
      case COMPONENT_TYPE.FILE:
        data = null;
        break;
      case COMPONENT_TYPE.MEMBER:
        data = getMemberValue(field, data);
        break;
      case COMPONENT_TYPE.AUTO_NUMBER:
        data = getAutoNumberValue(field, data);
        break;
      default:
        break;
    }
  }

  return {
    type,
    data,
  };
}

function numberOrder (a, b) {
  return a - b;
}

function stringOrder(a, b) {
  const reg = /[a-zA-Z0-9]/;
  if (reg.test(a) || reg.test(b)) {
    if (a > b) {
      return 1;
    } else if (a < b) {
      return -1;
    }
    return 0;
  } else {
    return a.localeCompare(b, 'zh-CN');
  }
}

function getSelectValue(field, value) {
  const op = field.options.find(o => o.value === value);
  if (!op) return null;
  return op.label;
}

function getMulipleSelectValue(field, value = []) {
  let result = '';
  for (const item of value) {
    const op = field.options.find(o => o.value === item);
    if (!op) continue;
    result += `${op.label}`;
  }
  return result;
}

function getDateValue(field, value) {
  let format = 'YYYY-MM-DD';
  if (field.dateFormat) format = field.dateFormat;
  return dayjs(value).format(format);
}

function getMemberValue(field, value) {
  let result = '';
  if (field.choice === 'single') {
    result = value[0]?.name ?? '';
  }
  if (field.choice === 'multiple') {
    let values = [];
    for (const item of value) {
      values.push(item.name || '');
    }
    values = values.sort((a, b) => {
      return stringOrder(a, b);
    });
    result = values.join('');
  }

  return result;
}

function getAutoNumberValue(field, value) {
  let result = '';
  if (typeof value === 'object') {
    result = value?.value ?? '';
  }
  if (typeof value === 'string') {
    result = value;
  }

  return result;
}

function orderParse(order, rule) {
  let result = order;
  if (rule.toLocaleLowerCase() === 'desc') {
    switch (order) {
      case -1:
        result = 1;
        break;
      case 1:
        result = -1;
        break;
      default: result = order;
        break;
    }
  }

  return result;
}

module.exports = {
  parseDatasOrder,
}
